syntax = "proto2";

package google.protobuf.compiler.tools;

// Non trivial child message
message AnalyzeChild {
  optional int32 child_id = 1;
  optional AnalyzeChild child = 2;
}

message AnalyzeThis {
  message Nested {
    optional int32 nexted_id = 1;
    optional string optional_string = 2;
  }

  optional int32 id = 1;
  optional string optional_string = 2;
  repeated string repeated_string = 3;
  optional AnalyzeChild optional_child = 4;
  repeated AnalyzeChild repeated_child = 5;
  optional Nested nested = 6;
}
