edition = "2023";

package proto2_unittest;

import "google/protobuf/late_loaded_option.proto";

message LateLoadedOptionUser {
  option (proto2_unittest.LateLoadedOption.ext) = {
    value: 1
  };
}
