// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: kenton@google.com (Kenton Varda)
//  Based on original Protocol Buffers design by
//  Sanjay Ghemawat, Jeff Dean, and others.
//
// This file contains messages for testing extensions.

syntax = "proto2";

package proto2_unittest;

option optimize_for = SPEED;
option csharp_namespace = "Google.ProtocolBuffers.TestProtos";

// A message with message_set_wire_format.
message TestExtensionSet {
  extensions 4 to max;
}

message TestExtensionSetContainer {
  optional TestExtensionSet extension = 1;
}
